@@ -13,8 +13,8 @@ has specification => 'https://spec.openapis.org/oas/3.0/schema/2019-04-02';
 # some methods are shared with OpenAPIv2
 monkey_patch __PACKAGE__,
   $_ => JSON::Validator::Schema::OpenAPIv2->can($_)
-  for qw(coerce validate_request validate_response),
-  qw(_coerce_arrays _coerce_default_value _find_all_nodes _prefix_error_path), qw(_validate_request_or_response);
+  for qw(coerce routes validate_request validate_response),
+  qw(_coerce_arrays _coerce_default_value _find_all_nodes _prefix_error_path _validate_request_or_response);
 
 sub new {
   my $self = shift->SUPER::new(@_);
@@ -256,6 +256,21 @@ sub _validate_body {
   return;
 }
 
+sub _validate_type_boolean {
+  my $self = shift;
+  return $_[2]->{nullable} && !defined $_[0] ? () : $self->SUPER::_validate_type_boolean(@_);
+}
+
+sub _validate_type_integer {
+  my $self = shift;
+  return $_[2]->{nullable} && !defined $_[0] ? () : $self->SUPER::_validate_type_integer(@_);
+}
+
+sub _validate_type_number {
+  my $self = shift;
+  return $_[2]->{nullable} && !defined $_[0] ? () : $self->SUPER::_validate_type_number(@_);
+}
+
 sub _validate_type_object {
   my ($self, $data, $path, $schema) = @_;
   return E $path, [object => type => data_type $data] if ref $data ne 'HASH';
@@ -272,15 +287,6 @@ sub _validate_type_object {
     return $self->_validate($data, $path, $self->get($url));
   }
 
-  my %properties = %{$schema->{properties} || {}};
-  local $schema->{properties} = \%properties;
-  for my $key (keys %properties) {
-    next unless $properties{$key}{nullable};
-    my $tied = tied %{$properties{$key}};
-    $properties{$key} = $tied ? {%{$tied->schema}} : {%{$properties{$key}}};
-    $properties{$key}{type} = ['null', _to_list($properties{$key}{type})];
-  }
-
   return $self->{validate_request}
     ? $self->_validate_type_object_request($_[1], $path, $schema)
     : $self->_validate_type_object_response($_[1], $path, $schema);
@@ -326,6 +332,11 @@ sub _validate_type_object_response {
   );
 }
 
+sub _validate_type_string {
+  my $self = shift;
+  return $_[2]->{nullable} && !defined $_[0] ? () : $self->SUPER::_validate_type_string(@_);
+}
+
 1;
 
 =encoding utf8
@@ -409,6 +420,12 @@ Example return value:
 
 The return value MUST not be mutated.
 
+=head2 routes
+
+  $collection = $schema->routes;
+
+Shares the same interface as L<JSON::Validator::Schema::OpenAPIv2/routes>.
+
 =head2 validate_request
 
   @errors = $schema->validate_request([$method, $path], \%req);
